import { __decorate } from "tslib";
import { Extension } from '@roots/bud-framework/extension';
import { bind, label } from '@roots/bud-framework/extension/decorators';
import { Config } from './config.js';
/**
 * Babel support for `@roots/bud`
 *
 * @public
 */
let BabelExtension = class BabelExtension extends Extension {
    /**
     * Babel cache directory
     *
     * @public
     */
    get cacheDirectory() {
        return this.app.path(`@storage/cache/babel`);
    }
    /**
     * Babel env
     *
     * @public
     */
    get env() {
        return {
            development: {
                compact: false,
            },
        };
    }
    /**
     * Root directory
     *
     * @public
     */
    get root() {
        return this.app.path();
    }
    /**
     * Babel RuleSetItem callback
     *
     * @public
     * @decorator `@bind`
     */
    setRuleSetItem(ruleSetItem) {
        return ruleSetItem.setLoader('babel').setOptions(() => ({
            cacheDirectory: this.cacheDirectory,
            presets: Object.values(this.app.babel.presets),
            plugins: Object.values(this.app.babel.plugins),
            env: this.env,
            root: this.root,
        }));
    }
    /**
     * Initialize extension
     *
     * @public
     * @decorator `@bind`
     */
    async init() {
        this.app.babel = new Config();
    }
    /**
     * Register extension
     *
     * @public
     * @decorator `@bind`
     */
    async register() {
        const loader = await this.resolve('babel-loader');
        const presetEnv = await this.resolve('@babel/preset-env');
        const restSpread = await this.resolve('@babel/plugin-proposal-object-rest-spread');
        const transformRuntime = await this.resolve('@babel/plugin-transform-runtime');
        const dynamicImport = await this.resolve('@babel/plugin-syntax-dynamic-import');
        const classProperties = await this.resolve('@babel/plugin-proposal-class-properties');
        this.app.build
            .setLoader('babel', loader)
            .setItem('babel', this.setRuleSetItem);
        this.app.build.rules.js.setUse(items => ['babel', ...items]);
        this.app.babel
            .setPresets({ '@babel/preset-env': presetEnv })
            .setPlugins({
            '@babel/plugin-transform-runtime': [
                transformRuntime,
                { helpers: false },
            ],
            '@babel/plugin-proposal-object-rest-spread': restSpread,
            '@babel/plugin-syntax-dynamic-import': dynamicImport,
            '@babel/plugin-proposal-class-properties': classProperties,
        });
    }
};
__decorate([
    bind
], BabelExtension.prototype, "setRuleSetItem", null);
__decorate([
    bind
], BabelExtension.prototype, "init", null);
__decorate([
    bind
], BabelExtension.prototype, "register", null);
BabelExtension = __decorate([
    label('@roots/bud-babel')
], BabelExtension);
export default BabelExtension;
//# sourceMappingURL=extension.js.map